@@ -12,8 +12,18 @@ from ldap.filter import escape_filter_chars
 
 logger = logging.getLogger(__name__)
 
+BASE_USER_FILTER = Filter('objectClass=user')
 
-USER_SEARCH_FILTER = Filter('objectClass=user') & Filter('objectCategory=Person') & ~Filter('sAMAccountName=krbtgt*')
+USER_SEARCH_FILTER = (
+    BASE_USER_FILTER
+    & Filter('objectCategory=Person')
+    & ~Filter('sAMAccountName=krbtgt*')
+)
+
+COMPUTER_SEARCH_FILTER = (
+    BASE_USER_FILTER
+    & Filter('objectCategory=Computer')
+)
 
 GROUP_SEARCH_FILTER = Filter('objectClass=group')
 
@@ -82,6 +92,13 @@ class ADManager:
         if scope is None:
             scope = ldap.SCOPE_SUBTREE
 
+        if isinstance(scope, str):
+            # Map scope string to ldap scope
+            scope = {
+                'subtree':  ldap.SCOPE_SUBTREE,
+                'one':      ldap.SCOPE_ONELEVEL,
+            }[scope]
+
         logger.debug("Search base: {}".format(base))
         logger.debug("Search scope: {}".format(scope))
         logger.debug("Search filter: {}".format(filt))
@@ -98,6 +115,15 @@ class ADManager:
             yield dn, attrs
 
 
+    def _get_objects(self, cls, rdn=None, filt=None, attrs=None, scope=None):
+        if attrs is None:
+            attrs = cls.default_ldap_attrs()
+
+        objlist = self._search(base_rdn=rdn, filt=filt, attrs=attrs, scope=scope)
+        for dn, attrvals in objlist:
+            yield cls(self, dn, **attrvals)
+
+
     def _modify(self, dn, modlist):
         rc = self.ldapconn.modify_s(dn, modlist)
         logger.debug("modify_s({!r}, {!r}) returned {}".format(
@@ -105,35 +131,28 @@ class ADManager:
 
 
     def get_users(self, rdn=None, filt=None, attrs=None, scope=None):
-        if attrs is None:
-            attrs = AdUser.default_ldap_attrs()
-
         f = USER_SEARCH_FILTER
         if filt is not None:    # Append caller filter
             f = f & filt
-
-        userlist = self._search(base_rdn=rdn, filt=f, attrs=attrs, scope=scope)
-        for dn, attrvals in userlist:
-            yield AdUser(self, dn, **attrvals)
-
+        return self._get_objects(AdUser, rdn=rdn, filt=f, attrs=attrs, scope=scope)
 
     def get_user_by_uid(self, uid, attrs=None):
         filt = Filter('uidNumber={}'.format(uid))
         return single_or(self.get_users(filt=filt, attrs=attrs), None)
 
 
-    def get_groups(self, rdn=None, filt=None, attrs=None):
-        if attrs is None:
-            attrs = AdGroup.default_ldap_attrs()
+    def get_computers(self, rdn=None, filt=None, attrs=None, scope=None):
+        f = COMPUTER_SEARCH_FILTER
+        if filt is not None:    # Append caller filter
+            f = f & filt
+        return self._get_objects(AdComputer, rdn=rdn, filt=f, attrs=attrs, scope=scope)
 
+
+    def get_groups(self, rdn=None, filt=None, attrs=None, scope=None):
         f = GROUP_SEARCH_FILTER
         if filt is not None:    # Append caller filter
             f = f & filt
-
-        grouplist = self._search(rdn, f, attrs)
-        for dn, attrvals in grouplist:
-            yield AdGroup(self, dn, **attrvals)
-
+        return self._get_objects(AdGroup, rdn=rdn, filt=f, attrs=attrs, scope=scope)
 
     def get_group_by_gid(self, gid, attrs=None):
         filt = Filter('gidNumber={}'.format(gid))
@@ -153,10 +172,13 @@ class AdUser(LdapObject):
         SIDAttr('objectSid'),
         StrAttr('userPrincipalName', writable=True),
         StrAttr('mail'),
+        StrAttr('sAMAccountName'),
         FILETIMEAttr('pwdLastSet'),
         FILETIMEAttr('PasswordExpiryTime', ldapattr='msDS-UserPasswordExpiryTimeComputed'),
     )
 
+class AdComputer(AdUser):
+    pass
 
 class AdGroup(LdapObject):
     _known_attrs = (
