@@ -1,4 +1,5 @@
 import argparse
+from datetime import datetime
 import logging
 import sys
 from functools import wraps
@@ -6,6 +7,7 @@ from functools import wraps
 from .adman import ADManager
 from .assign import PosixIdAssigner, AssignmentError, UpnAssigner
 from .config import Config, ConfigError
+from .userdir import UserdirCreator, TemplatedSecurityDescriptor
 from .paths import default_confpath
 from .pwexpire import check_pw_expiry
 from .state import DomainState
@@ -19,6 +21,9 @@ logger = logging.getLogger(__name__)
 class AppError(Exception):
     pass
 
+class NotConfiguredError(AppError):
+    def __init__(self, cfgsec):
+        super().__init__("Section not configured: " + cfgsec)
 
 
 def confirm(question="Are you sure?"):
@@ -42,20 +47,45 @@ def get_state(ad):
     return s
 
 
-def get_posix_assigner(ad, args):
-    def change_handler(message, data):
-        args.changelog.write(message + '\n')
+class ChangeHandler:
+    def __init__(self, logfile):
+        self.logfile = logfile
+
+    def __call__(self, message):
+        self.logfile.write("{:%Y-%m-%d %H:%M:%S} {}\n"
+                           .format(datetime.now(), message))
+
+
+def get_change_handler(config):
+    path = config.changelog_path
+    f = open(path, 'a') if path else sys.stderr
+    return ChangeHandler(f)
+
+
+def get_posix_assigner(args, ad):
+    ia = args.config.id_assign
+    if ia is None:
+        raise NotConfiguredError("id_assign")
 
     return PosixIdAssigner(
             ad = ad,
             state = get_state(ad),
-            uid_range = args.config.uid_range,
-            gid_range = args.config.gid_range,
+            uid_range = ia.uid_range,
+            gid_range = ia.gid_range,
             replace_invalid = args.force,
-            onchange = change_handler,
+            onchange = get_change_handler(args.config),
             )
 
 
+def posix_assign(func):
+    @wraps(func)
+    @connect_ad
+    def wrapper(args, ad):
+        pa = get_posix_assigner(args, ad)
+        return func(args, ad, pa)
+    return wrapper
+
+
 def connect_ad(func):
     @wraps(func)
     def wrapper(args):
@@ -75,74 +105,101 @@ def connect_ad(func):
             )
 
         ad = ADManager.connect(domain)
-        return func(ad, args)
+        return func(args, ad)
     return wrapper
 
 
 @connect_ad
-def cmd_allmaint(ad, args):
-    cmd_assignids.__wrapped__(ad, args)
-    cmd_user_setupns.__wrapped__(ad, args)
-    cmd_user_checkexpire.__wrapped__(ad, args)
+def cmd_allmaint(args, ad):
+    tasks = [
+        cmd_assignids,
+        cmd_user_setupns,
+        cmd_user_checkexpire,
+        cmd_user_mkdirs,
+    ]
+
+    for cmd in tasks:
+        try:
+            cmd.__wrapped__(args, ad)
+        except NotConfiguredError as e:
+            logger.info(str(e))
+
 
 
 @connect_ad
-def cmd_assignids(ad, args):
+def cmd_assignids(args, ad):
+    pa = get_posix_assigner(args, ad)
+
     logger.info("Assigning POSIX IDs")
-    a = get_posix_assigner(ad, args)
-    a.assign_group_gidNumbers()
-    a.assign_user_uidNumbers()
-    a.update_user_gidNumbers()
+    cmd_group_assign.__wrapped__(args, ad, pa)
+    cmd_user_assign.__wrapped__(args, ad, pa)
+    if args.config.id_assign.computers:
+        cmd_computer_assign.__wrapped__(args, ad, pa)
+    else:
+        logger.info("Not assigning computer IDs")
 
 
-@connect_ad
-def cmd_clearids(ad, args):
+@posix_assign
+def cmd_clearids(args, ad, pa):
     if not args.force:
         print("This command will clear *all* uidNumber/gidNumber attributes!")
         if not confirm():
             return
 
-    a = get_posix_assigner(ad, args)
-    a.clear_group_gidNumbers()
-    a.clear_user_uidNumbers()
-    a.clear_user_gidNumbers()
+    pa.clear_all_ids()
+
+
+@posix_assign
+def cmd_computer_assign(args, ad, pa):
+    ia = args.config.id_assign
+    for container, scope in ia.only.iterate_containers():
+        pa.computer_assign(container=container, scope=scope)
 
 
 @connect_ad
-def cmd_group_assign(ad, args):
-    a = get_posix_assigner(ad, args)
-    a.assign_group_gidNumbers()
+def cmd_computer_list(args, ad):
+    common_list(ad.get_computers(), args.verbose)
+
+
+@posix_assign
+def cmd_group_assign(args, ad, pa):
+    ia = args.config.id_assign
+    for container, scope in ia.only.iterate_containers():
+        pa.assign_group_gidNumbers(
+                container = container,
+                scope = scope,
+                include_computer_groups = ia.computers,
+            )
 
 
 @connect_ad
-def cmd_group_list(ad, args):
+def cmd_group_list(args, ad):
     common_list(ad.get_groups(), args.verbose)
 
 
-@connect_ad
-def cmd_user_assign(ad, args):
-    a = get_posix_assigner(ad, args)
-    a.assign_user_uidNumbers()
-    a.update_user_gidNumbers()
+@posix_assign
+def cmd_user_assign(args, ad, pa):
+    ia = args.config.id_assign
+    for container, scope in ia.only.iterate_containers():
+        pa.user_assign(container=container, scope=scope)
 
 
 @connect_ad
-def cmd_user_checkexpire(ad, args):
+def cmd_user_checkexpire(args, ad):
     cfg = args.config
     if not cfg.pwexp:
-        logger.info("Password expiry notification not configured")
-        return
+        raise NotConfiguredError("password_expiry_notification")
     check_pw_expiry(ad, cfg)
 
 
 @connect_ad
-def cmd_user_setupns(ad, args):
-    def change_handler(message, data):
-        args.changelog.write(message + '\n')
+def cmd_user_setupns(args, ad):
+    if not args.config.upn_suffixes:
+        raise NotConfiguredError("upn_suffixes")
 
     a = UpnAssigner(
         ad = ad,
-        onchange = change_handler,
+        onchange = get_change_handler(args.config),
     )
 
     logger.info("Settings UPNs")
@@ -150,6 +207,39 @@ def cmd_user_setupns(ad, args):
         a.set_user_upn_suffixes(container, cfg.suffix, cfg.scope)
 
 
+@connect_ad
+def cmd_user_mkdirs(args, ad):
+    creator = UserdirCreator(
+        ad = ad,
+        onchange = get_change_handler(args.config),
+    )
+
+    for ud in args.config.userdirs:
+        tsd = TemplatedSecurityDescriptor(
+                owner = ud.owner,
+                group = ud.group,
+                acl = ud.acl,
+            )
+
+        for container, scope in ud.only.iterate_containers():
+            creator.create_userdirs(
+                basepath = ud.basepath,
+                tsd = tsd,
+                container = container,
+                scope = scope,
+                subdirs = [
+                    (
+                        sd.name,
+                        TemplatedSecurityDescriptor(
+                            owner = ud.owner,   # inherit
+                            group = ud.group,   # inherit
+                            acl = sd.acl,
+                            )
+                    )
+                    for sd in ud.subdirs
+                ],
+            )
+
 
 def common_list(things, verbose):
     for x in things:
@@ -159,18 +249,18 @@ def common_list(things, verbose):
             print(x.cn)
 
 @connect_ad
-def cmd_user_list(ad, args):
+def cmd_user_list(args, ad):
     common_list(ad.get_users(), args.verbose)
 
 
 @connect_ad
-def cmd_state_list(ad, args):
+def cmd_state_list(args, ad):
     s = get_state(ad)
     print(s)
 
 
 @connect_ad
-def cmd_state_init(ad, args):
+def cmd_state_init(args, ad):
     ds = DomainState.get(ad)
 
     logger.info("Got DomainState ({}): {}".format(ds.dn, ds))
@@ -212,9 +302,10 @@ def cmd_state_init(ad, args):
         logger.info("Setting Domain state {} to {}".format(ds_attr, useval))
         setattr(ds, ds_attr, useval)
 
+    ia = args.config.id_assign
 
-    set_next_xid(ad.get_users(),  'user',  'uidNumber', 'next_uid', args.config.uid_range),
-    set_next_xid(ad.get_groups(), 'group', 'gidNumber', 'next_gid', args.config.gid_range),
+    set_next_xid(ad.get_users(),  'user',  'uidNumber', 'next_uid', ia.uid_range),
+    set_next_xid(ad.get_groups(), 'group', 'gidNumber', 'next_gid', ia.gid_range),
 
     ds.commit()
     print(ds)
@@ -231,8 +322,6 @@ def parse_args():
     ap.add_argument('--loglevel', type=str.upper, default="warning",
             choices=("DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"),
             help="Log level")
-    ap.add_argument('--changelog', type=argparse.FileType('a'), default=sys.stderr,
-            help="File to which changes are logged")
     ap.add_argument('-c', '--config', default=default_confpath(),
             help="Path to config file")
     ap.add_argument('-v', '--verbose', action='store_true',
@@ -265,6 +354,25 @@ def parse_args():
     p.add_argument('--force', action='store_true',
             help="Forcefully clear uidNumber/gidNumber attributes")
 
+    ######
+    # computer
+    p_computer = sub.add_parser('computer',
+            help="Computer sub-commands")
+    sp_computer = p_computer.add_subparsers(metavar='SUBCOMMAND')
+    sp_computer.required = True
+
+    # computer assign
+    p = sp_computer.add_parser('assign',
+            help="Assign missing uidNumber attributes")
+    p.set_defaults(func=cmd_computer_assign)
+    add_assign_opts(p)
+
+    # computer list
+    p = sp_computer.add_parser('list',
+            help="List computers")
+    p.set_defaults(func=cmd_computer_list)
+
+
     ######
     # group
     p_group = sub.add_parser('group',
@@ -335,6 +443,11 @@ def parse_args():
             help="List users")
     p.set_defaults(func=cmd_user_list)
 
+    # user mkdirs
+    p = sp_user.add_parser('mkdirs',
+            help="Make user directories")
+    p.set_defaults(func=cmd_user_mkdirs)
+
 
     return ap.parse_args()
 
