@@ -93,7 +93,10 @@ def load_cred_cache(ccache):
         '-c', str(ccache),
     ]
     logger.debug("Running `{}`".format(' '.join(args)))
-    cp = subprocess.run(args, check=True, capture_output=True, universal_newlines=True)
+    cp = subprocess.run(args, check=True,
+                        stdout=subprocess.PIPE,
+                        stderr=subprocess.PIPE,
+                        universal_newlines=True)
 
     result = CredCache()
 
@@ -180,7 +183,7 @@ def check_cred_cache(cache, realm, mintime=None):
     except subprocess.CalledProcessError:
         return False
 
-    spn = make_principal('krbtgt', realm, realm)
+    spn = make_principal('krbtgt', realm, realm.upper())
     try:
         issued_at, expire_at = cc.tickets[spn]
     except KeyError:
